summaryrefslogtreecommitdiff
path: root/contrib/twirllib/src/TwirlWorker.scala
diff options
context:
space:
mode:
Diffstat (limited to 'contrib/twirllib/src/TwirlWorker.scala')
-rw-r--r--contrib/twirllib/src/TwirlWorker.scala79
1 files changed, 49 insertions, 30 deletions
diff --git a/contrib/twirllib/src/TwirlWorker.scala b/contrib/twirllib/src/TwirlWorker.scala
index 09376a6f..19eb4725 100644
--- a/contrib/twirllib/src/TwirlWorker.scala
+++ b/contrib/twirllib/src/TwirlWorker.scala
@@ -4,6 +4,7 @@ package twirllib
import java.io.File
import java.lang.reflect.Method
import java.net.URLClassLoader
+import java.nio.charset.Charset
import mill.api.PathRef
import mill.scalalib.api.CompilationResult
@@ -22,24 +23,29 @@ class TwirlWorker {
// Switched to using the java api because of the hack-ish thing going on later.
//
// * we'll need to construct a collection of additional imports
// * it will need to consider the defaults
// * and add the user-provided additional imports
// * the default collection in scala api is a Seq[String]
// * but it is defined in a different classloader (namely in cl)
// * so we can not construct our own Seq and pass it to the method - it will be from our classloader, and not compatible
- // * the java api has a Collection as the type for this param, for which it is much more doable to append things to it using reflection
+ // * the java api uses java collections, manipulating which using reflection is much simpler
//
- // NOTE: I tried creating the cl classloader passing the current classloader as the parent:
+ // NOTE: When creating the cl classloader with passing the current classloader as the parent:
// val cl = new URLClassLoader(twirlClasspath.map(_.toIO.toURI.toURL).toArray, getClass.getClassLoader)
- // in that case it was possible to cast the default to a Seq[String], construct our own Seq[String], and pass it to the method invoke- it was compatible.
- // And the tests passed. But when run in a different mill project, I was getting exceptions like this:
+ // it is possible to cast the default to a Seq[String], construct our own Seq[String], and pass it to the method invoke -
+ // classe will be compatible (the tests passed).
+ // But when run in an actual mill project with this module enabled, there were exceptions like this:
// scala.reflect.internal.MissingRequirementError: object scala in compiler mirror not found.
val twirlCompilerClass = cl.loadClass("play.japi.twirl.compiler.TwirlCompiler")
- // this one is only to get the codec: Codec parameter default value
- val twirlScalaCompilerClass = cl.loadClass("play.twirl.compiler.TwirlCompiler")
+ val codecClass = cl.loadClass("scala.io.Codec")
+ val charsetClass = cl.loadClass("java.nio.charset.Charset")
+ val arrayListClass = cl.loadClass("java.util.ArrayList")
+ val hashSetClass = cl.loadClass("java.util.HashSet")
+
+ val codecApplyMethod = codecClass.getMethod("apply", charsetClass)
+ val charsetForNameMethod = charsetClass.getMethod("forName", classOf[java.lang.String])
val compileMethod = twirlCompilerClass.getMethod("compile",
classOf[java.io.File],
@@ -51,11 +57,9 @@ class TwirlWorker {
cl.loadClass("scala.io.Codec"),
classOf[Boolean])
- val arrayListClass = cl.loadClass("java.util.ArrayList")
- val hashSetClass = cl.loadClass("java.util.HashSet")
+ val defaultImportsMethod = twirlCompilerClass.getField("DEFAULT_IMPORTS")
- val defaultAdditionalImportsMethod = twirlCompilerClass.getField("DEFAULT_IMPORTS")
- val defaultCodecMethod = twirlScalaCompilerClass.getMethod("compile$default$7")
+ val hashSetConstructor = hashSetClass.getConstructor(cl.loadClass("java.util.Collection"))
val instance = new TwirlWorkerApi {
override def compileTwirl(source: File,
@@ -66,27 +70,42 @@ class TwirlWorker {
constructorAnnotations: Seq[String],
codec: Codec,
inclusiveDot: Boolean) {
- val defaultAdditionalImports = defaultAdditionalImportsMethod.get(null) // unmodifiable collection
- // copying it into a modifiable hash set and adding all additional imports
- val allAdditionalImports =
- hashSetClass
- .getConstructor(cl.loadClass("java.util.Collection"))
- .newInstance(defaultAdditionalImports)
- .asInstanceOf[Object]
- val hashSetAddMethod =
- allAdditionalImports
- .getClass
- .getMethod("add", classOf[Object])
- additionalImports.foreach(hashSetAddMethod.invoke(allAdditionalImports, _))
-
+ // val defaultImports = play.japi.twirl.compiler.TwirlCompiler.DEFAULT_IMPORTS()
+ // val twirlAdditionalImports = new HashSet(defaultImports)
+ // additionalImports.foreach(twirlAdditionalImports.add)
+ val defaultImports = defaultImportsMethod.get(null) // unmodifiable collection
+ val twirlAdditionalImports = hashSetConstructor.newInstance(defaultImports).asInstanceOf[Object]
+ val hashSetAddMethod = twirlAdditionalImports.getClass.getMethod("add", classOf[Object])
+ additionalImports.foreach(hashSetAddMethod.invoke(twirlAdditionalImports, _))
+
+ // Codec.apply(Charset.forName(codec.charSet.name()))
+ val twirlCodec = codecApplyMethod.invoke(null, charsetForNameMethod.invoke(null, codec.charSet.name()))
+
+ // val twirlConstructorAnnotations = new ArrayList()
+ // constructorAnnotations.foreach(twirlConstructorAnnotations.add)
+ val twirlConstructorAnnotations = arrayListClass.newInstance().asInstanceOf[Object]
+ val arrayListAddMethod = twirlConstructorAnnotations.getClass.getMethod("add", classOf[Object])
+ constructorAnnotations.foreach(arrayListAddMethod.invoke(twirlConstructorAnnotations, _))
+
+ // JavaAPI
+ // public static Optional<File> compile(
+ // File source,
+ // File sourceDirectory,
+ // File generatedDirectory,
+ // String formatterType,
+ // Collection<String> additionalImports,
+ // List<String> constructorAnnotations,
+ // Codec codec,
+ // boolean inclusiveDot
+ // )
val o = compileMethod.invoke(null, source,
sourceDirectory,
generatedDirectory,
formatterType,
- allAdditionalImports,
- arrayListClass.newInstance().asInstanceOf[Object], // empty list seems to be the default
- defaultCodecMethod.invoke(null),
- Boolean.box(false)
+ twirlAdditionalImports,
+ twirlConstructorAnnotations,
+ twirlCodec,
+ Boolean.box(inclusiveDot)
)
}
}
@@ -22,24 +23,29 @@ class TwirlWorker {
// Switched to using the java api because of the hack-ish thing going on later.
//
- // * we'll need to construct a collection of additional imports
- // * it will need to consider the defaults
- // * and add the user-provided additional imports
+ // * we'll need to construct a collection of additional imports (will need to also include the defaults and add the user-provided additional imports)
+ // * we'll need to construct a collection of constructor annotations// *
// * the default collection in scala api is a Seq[String]
// * but it is defined in a different classloader (namely in cl)